#!/usr/bin/python3
import requests
import argparse
from datetime import datetime, timedelta
import pytz
import pyrfc3339
import traceback
import csv
from io import StringIO

UNKNOWN = -1
OK = 0
WARNING = 1
CRITICAL = 2

def parse_args():
    parser = argparse.ArgumentParser(description="Performs a query to an influx database according to https://docs.influxdata.com/influxdb/cloud/query-data/execute-queries/influx-api/")
    parser.add_argument("-w", type=float, default=1800, help="age in seconds")
    parser.add_argument("-c", type=float, default=600, help="age in seconds")
    parser.add_argument("-t", "--token", type=str, required=True)
    parser.add_argument("-o", "--org-id", type=str, required=True)
    parser.add_argument("-H", "--host", type=str, default="localhost")
    parser.add_argument("-p", "--port", type=int, default=8086)
    parser.add_argument("--query", type=str, required=True, 
                        help="query must return a CSV content with header and one column labeld '_time' in RFC3339. "
                             "Timestamp is taken as the most recent value over all returned lines.")
    parser.add_argument("-ssl", action="store_true", default=False, help="Use https instead of http")
    parser.add_argument("-v", "--verbose", action="store_true", default=False)

    return parser.parse_args()

if __name__=="__main__":
    try:
        args=parse_args()

        protocol="https" if args.ssl else "http"

        now = datetime.now(pytz.utc)
        datetime_warn = now - timedelta(seconds=args.w)
        datetime_crit = now - timedelta(seconds=args.c)

        URL=f"{protocol}://{args.host}:{args.port}/api/v2/query?orgID={args.org_id}"
        data=args.query.encode()
        headers={'Authorization': f'Token {args.token}',
                                     'Accept': 'application/csv',
                                     'Content-type': 'application/vnd.flux'}
        
        if args.verbose:
            print(f"datetime_warn: {datetime_warn}")
            print(f"datetime_crit: {datetime_crit}")
            print(f"URL: {URL}")
            print(f"data: {data}")
            print(f"headers: {headers}")

        if datetime_warn > datetime_crit:
            print("UNKOWN - Timedelta for warning must be larger than timedelta for critical")
            exit(UNKNOWN)
                  
        req = requests.post(URL, data=data, headers=headers)

        if req.ok:
            if args.verbose:
                print("Response:", req.text)
            rows = [row for row in csv.reader(StringIO(req.text))]
    
            #print(rows)
            if len(rows) < 2:
                print('UNKNOWN - query retured no data')
                exit(UNKNOWN)

            # get index of column containing time data
            col_idx = -1
            for idx_col, col in enumerate(rows[0]):
                if col == '_time':
                    col_idx = idx_col
                    break
            assert col_idx >0

            # parse timestamps of all returned tables and lines
            timestamps = [ pyrfc3339.parse(row[col_idx]) for row in rows[1:] if len(row)>col_idx ]
            time_of_last_entry = max(timestamps)
            if args.verbose:
                print(timestamps, time_of_last_entry)

            age = (now - time_of_last_entry).total_seconds()
            text = f"time of last entry: {time_of_last_entry}, age: {age}s|age={age}s;{args.w};{args.c};;"
            if time_of_last_entry < datetime_crit:
                print(f"CRIT - {text}")
                exit(CRITICAL)
            elif time_of_last_entry < datetime_warn:
                print(f"WARN - {text}")
                exit(WARNING)
            else:
                print(f"OK - {text}")
        else:
              print("UNKNOWN - response was:", req.__dict__)
              exit(UNKNOWN)

    except Exception as e:
        print(f"UNKNOWN - {traceback.format_exception(None, e, e.__traceback__)}")
        exit(UNKNOWN)

